/* The copyright in this software is being made available under the BSD
 * License, included below. This software may be subject to other third party
 * and contributor rights, including patent rights, and no such rights are
 * granted under this license.  
 *
 * Copyright (c) 2010-2011, ITU/ISO/IEC
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *  * Neither the name of the ITU/ISO/IEC nor the names of its contributors may
 *    be used to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

/** \file     TEncAdaptiveLoopFilter.h
 \brief    estimation part of adaptive loop filter class (header)
 */

#ifndef __TENCLOOPFILTER__
#define __TENCLOOPFILTER__

#include "TLibCommon/TComLoopFilter.h"
#include "TLibCommon/TComPic.h"

#if ADAPT_DF_OFFSET
#include "TEncEntropy.h"
#include "../TLibCommon/TComBitCounter.h"
#endif


//! \ingroup TLibEncoder
//! \{
#if ADAPT_DF_OFFSET
// ====================================================================================================================
// Class definition
// ====================================================================================================================
#define MAX_OFFSET    20  //Maximum offset for DF paramemters from 0 value


class TEncLoopFilter : public TComLoopFilter
{

  protected:

  TEncEntropy*       m_pcEntropyCoder;

  TComPic*     m_pcPic;
  TComPicYuv   m_pcTempPicYuv;

  Int          m_iBetaRange;
  Int          m_iTcRange;
  Int          m_iSideBetaRange;

  Int          m_iBestBeta;
  Int          m_iBestTc;
  Int          m_iBestSideBeta;


  UInt64       m_iCounter[MAX_OFFSET*2+1][MAX_OFFSET*2+1];   // beta, beta/6, 

  Int m_iBestBetaOffset ;           
  Int m_iBestSideBetaOffset ;
  Int m_iBestTcOffset ;


  UInt         m_uiSSDStrong; 
  UInt         m_uiSSDWeak; 
  UInt         m_uiSSDOrig; 
  UInt         m_uiSSDCenter; 
  UInt         m_uiSSD2PelP; 
  UInt         m_uiSSD2PelQ; 

  UInt         m_uiOptPass;


  public:

  TEncLoopFilter();
  ~TEncLoopFilter();  


  /// set entropy coder
  Void startFindLoopFilterParamsEnc(TEncEntropy* pcEntropyCoder);
  
  /// null the entropy coder
  Void endFindLoopFilterParamsEnc();

  Void findLoopFilterParamsFast( TComPic* pcPic, Int BetaRange, Int TcRange, Int SideBetaRange);
  
  Void loopFilterEstimatePic( TComPic* pcPic );

  protected:
#if PARALLEL_MERGED_DEBLK
  Void xEstimateCU                 ( TComDataCU* pcCU, UInt uiAbsZorderIdx, UInt uiDepth, Int Edge );
#else
  Void xEstimateCU                 ( TComDataCU* pcCU, UInt uiAbsZorderIdx, UInt uiDepth );
#endif
  
#if (PARALLEL_DEBLK_DECISION && !PARALLEL_MERGED_DEBLK)
  Void xEdgeEstimateLuma            ( TComDataCU* pcCU, UInt uiAbsZorderIdx, UInt uiDepth, Int iDir, Int iEdge, Int iDecideExecute);
#else
  Void xEdgeEstimateLuma            ( TComDataCU* pcCU, UInt uiAbsZorderIdx, UInt uiDepth, Int iDir, Int iEdge );
#endif

  

__inline Void estimateBlock(Pel* piTmpSrc, Pel* piTmpSrcJudge, Int iSrcStep, Int iIdx, UInt uiPelsInPart, Int iBlkIdx, Int iOffset, UInt uiTcOffset, Int iBitdepthScale, Int iQP  );

__inline Void estimateTc(Pel* piTmpSrc, Pel* piTmpSrcJudge, Int iSrcStep, Int iIdx, UInt uiPelsInPart, Int iBlkIdx, Int iOffset, UInt uiTcOffset, Int iBitdepthScale, Int iQP  );

__inline Int xCalcThrCut( Pel* piSrc, Int iOffset);

__inline UInt xSSDPelOrig( Pel* piSrc, Pel* piOrig, Int iOffset );

__inline UInt xSSDPelWeakFilt(  Pel* piSrc, Pel* piOrig, Int iOffset, UInt& ruiSSD2PelP, UInt& ruiSSD2Q, UInt& ruiSSDCenter, Int tc);

__inline UInt xSSDPelStrongFilt(  Pel* piSrc, Pel* piOrig, Int iOffset );

__inline Int xCalcDStrong( Pel* piSrc, Int iOffset);

__inline Int xCalcStrongMiddle( Pel* piSrc, Int iOffset);

__inline UInt xCalcSSDFilterEdge(Int beta, Int tc, Int ThrCut, Int iSideThr, Int d, Int d0, Int d3, Int dp, Int dq, Int iNaturalEdge0, Int iNaturalEdge3, Int iStrongMiddle0, Int iStrongMiddle3, Int iDStrong0, Int iDStrong3); 

__inline Void xCalcSSDFilterEdgeRealTc(Pel* piTmpSrc, Int iSrcStep, Int iOffset, Int iShift, Pel* piTmpSrcJudge, Int d, Int d0, Int  d3, Int dp, Int dq, Int beta, Int tc,  Int iThrCut, Int iSideThr, Int iBitdepthScale);

__inline Void xPelFilterLumaSSDTc( Pel* piSrc, Int iOffset, Int d, Int beta, Int tc , Pel* piSrcJudge, Int iThrCut, Bool bFilterSecondP, Bool bFilterSecondQ, Int iBitdepthScale);

Int findBetaOffset( Int iOffset, Int iQP, Int iDiv);

Int findTcOffset( Int iOffset, Int iQP, Int iDiv);

Int findSideBetaOffset( Int iOffset, Int iQP, Int iDiv);

};
#endif
#endif


 